Object subclass: #Stack
	instanceVariableNames: 'store top '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stacks'!

!Stack methodsFor: 'initialization' stamp: 'ab 4/28/2002 10:57'!
setsize: n
   store _ Array new: n.
   top _ 0.
! !


!Stack methodsFor: 'stack operations' stamp: 'ab 4/28/2002 10:53'!
isEmpty
   ^ top=0
! !

!Stack methodsFor: 'stack operations' stamp: 'ab 4/28/2002 10:57'!
pop | item | 
   self isEmpty ifTrue: [self error: 'trying to pop an empty stack'].
   item _ store at: top.
   top _ top-1.
   ^ item
! !

!Stack methodsFor: 'stack operations' stamp: 'ab 4/28/2002 10:57'!
push: item
   | oldStore |
   top _ top+1.
   top > store size ifTrue: 
      "store is about to overflow.  make a new array twice as big, and
       copy the old values into it"
      [oldStore _ store.
       store _ Array new: 2 * oldStore size.
       1 to: oldStore size do:
          [:k | store at: k put: (oldStore at: k)]].
   store at: top put: item
! !

!Stack methodsFor: 'stack operations' stamp: 'ab 4/28/2002 11:10'!
reset 
	top _ 0
! !


!Stack methodsFor: 'printing' stamp: 'ab 4/28/2002 11:07'!
printOn: strm
	self isEmpty ifTrue: [strm nextPutAll: 'empty stack'.  ^self].
	strm nextPutAll: 'stack with '.
	1 to: top-1 do: [:i | (store at: i) printOn: strm.  strm nextPutAll: ', '].
	(store at: top) printOn: strm.! !


!Stack methodsFor: 'testing' stamp: 'ab 4/28/2002 11:18'!
test
	"run some test cases on stacks, printing the results to the transcript"
	"Stack new test"
	| s1 s2 |
	Transcript cr; cr; show: 'running tests on Stack'; cr.
	s1 _ Stack new setsize: 10.
	s2 _ Stack new setsize: 10.
	Transcript show: s1; cr.
	s1 push: 3.14.  Transcript show: s1; cr.
	s1 push: 'fred'.  Transcript show: s1; cr.
	s2 push: 42.  Transcript show: s2; cr.
	s2 reset.  Transcript show: s2; cr.

	"the following pops should succeed, and print the results to the transcript"

	Transcript show: s1 pop; cr.
	Transcript show: s1 pop; cr.

	"finally, this should cause an error"
	s1 pop



! !


Stack subclass: #PoppingStack
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stacks'!

!PoppingStack methodsFor: 'iterating' stamp: 'ab 4/28/2002 11:13'!
popAndDo: block
	"pop each element of this stack and evaluate the block with the newly popped element"
	[self isEmpty] whileFalse: [block value: self pop]
! !


!PoppingStack methodsFor: 'testing' stamp: 'ky 5/11/2002 16:17'!
test
	"run a test case on popping stack, printing the results to the transcript"
		"PoppingStack new test"
	| p |
	Transcript cr; cr; show: 'running tests on PoppingStack'; cr.
	p _ PoppingStack new setsize: 10.
	Transcript show: 'empty stack w/ size 10:  '; show: p; cr.
	p push: 3.14.  
	p push: 'fred'. 
	p push: true.
	Transcript show: 'stack after 3 pushes:  '; show: p; cr.
	Transcript show: 'testing popAndDo:'; cr.
	p popAndDo: [:e | Transcript show: 'popping '; show: e; cr]


! !
